package jp.hasc.hasctool.ui.bdeditor2.parts;

import jp.hasc.hasctool.ui.bdeditor2.model.BlockDiagramElement;
import jp.hasc.hasctool.ui.bdeditor2.model.BlockElement;
import jp.hasc.hasctool.ui.bdeditor2.model.commands.BlockCreateCommand;
import jp.hasc.hasctool.ui.bdeditor2.model.commands.BlockSetConstraintCommand;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.ResizableEditPolicy;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.requests.CreateRequest;

/**
 * EditPolicy for the Figure used by this edit part. Children of
 * XYLayoutEditPolicy can be used in Figures with XYLayout.
 * 
 * @author Elias Volanakis
 */
class DiagramXYLayoutEditPolicy extends XYLayoutEditPolicy {

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * ConstrainedLayoutEditPolicy#createChangeConstraintCommand(ChangeBoundsRequest
	 * , EditPart, Object)
	 */
	protected Command createChangeConstraintCommand(
			ChangeBoundsRequest request, EditPart child, Object constraint) {
		if (child instanceof BlockEditPart && constraint instanceof Rectangle) {
			// return a command that can move and/or resize a Shape
			return new BlockSetConstraintCommand((BlockElement) child.getModel(),
					request, (Rectangle) constraint);
		}
		return super.createChangeConstraintCommand(request, child, constraint);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ConstrainedLayoutEditPolicy#createChangeConstraintCommand(EditPart,
	 * Object)
	 */
	protected Command createChangeConstraintCommand(EditPart child,
			Object constraint) {
		// not used in this example
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see LayoutEditPolicy#getCreateCommand(CreateRequest)
	 */
	protected Command getCreateCommand(CreateRequest request) {
		Object childClass = request.getNewObjectType();
		
		if (childClass.equals(BlockElement.class)) {
			return new BlockCreateCommand(this,
					(BlockElement) request.getNewObject(),
					(BlockDiagramElement) getHost().getModel(),
					(Rectangle) getConstraintFor(request));
		}
		
		return null;
	}

	@Override
	protected EditPolicy createChildEditPolicy(EditPart child) {
		if (child instanceof BlockEditPart) {
			ResizableEditPolicy policy = new ResizableEditPolicy();
			policy.setResizeDirections(0); // fixed size
			return policy;
		}

		// default
		return super.createChildEditPolicy(child);
	}

}